///////////////////////////////////////////////////////////////////////////////
// Decorator.h: Classes for illustrate Decorator pattern					 //
// Author: Tianyu Qi, tqi100@syr.edu										 //	
///////////////////////////////////////////////////////////////////////////////
#pragma once
#ifndef DECORATOR_H
#define DECORATOR_H
#include <iostream>
#include <string>

// Cake: Act as the component class
class Cake {
public:
	virtual std::string getCake() {
		return description;
	}
	virtual int total() = 0;
	virtual int single() = 0;

protected:
	std::string description;
};

// Brownie: Act as one of concrete component class
class Brownie : public Cake {
public:
	Brownie() {
		description = "A Brownie cake with lots of calories.";
	}
	int total() { return 10; }
	int single() { return 10; }
};

// Mousse: Act as one of concrete component class
class Mousse : public Cake {
public:
	Mousse() {
		description = "A Mousse cake with lots of calories.";
	}
	int total() { return 20; }
	int single() { return 20; }
};

// BlackForest: Act as one of concrete component class
class BlackForest : public Cake {
public:
	BlackForest() {
		description = "A Black Forest cake with lots of calories.";
	}
	int total() { return 30; }
	int single() { return 30; }
};

// Decorator: Act as Decorator class
class Decorator : public Cake {
protected:
	Cake * cake;
};

// ChocolateChip: Act as one of concrete decorator class
class ChocolateChip : public Decorator {
public:
	ChocolateChip(Cake *cake) {
		this->cake = cake;
	}
	std::string getCake() {
		return cake->getCake() + "\n  With chocolate chip added and more calories.";
	}
	int total() { return cake->total() + 1; }
	int single() { return 1; }
};

// Raisin: Act as one of concrete decorator class
class Raisin : public Decorator {
public:
	Raisin(Cake *cake) {
		this->cake = cake;
	}
	std::string getCake() {
		return cake->getCake() + "\n  With raisin added and more calories.";
	}
	int total() { return cake->total() + 2; }
	int single() { return 2; }
};

// MountCream: Act as one of concrete decorator class
class MountCream : public Decorator {
public:
	MountCream(Cake *cake) {
		this->cake = cake;
	}
	std::string getCake() {
		return cake->getCake() + "\n  With mount cream added and more calories.";
	}
	int total() { return cake->total() + 4; }
	int single() { return 4; }
};

#endif // !DECORATOR_H

