#pragma once
#include <iostream>
#include <string>

namespace sable {
	class Secret {
	public:
		virtual std::string getSecret() { return secret; }
		virtual std::string getPCR(int PCRNum) { return ""; }
		virtual int getLocality() { return -1; }
		virtual std::string getAuthValue() { return ""; }
	protected:
		std::string secret;
		std::string authValue;
		std::string PCR17;
		std::string PCR19;
		int locality;
	};

	// SECsecret: SABLE-Enable-Config object, protected by policy authorizations
	class SECsecret : public Secret {
	public:
		SECsecret(const std::string& passPhrase, 
				const std::string& setAuthValue, 
				const std::string& setPCR17, 
				const std::string& setPCR19, 
				int setLocality) {
			secret = passPhrase;
			authValue = setAuthValue;
			PCR17 = setPCR17;
			PCR19 = setPCR19;
			locality = setLocality;
		}
		std::string getPCR(int PCRNum) {
			if (PCRNum == 17) return PCR17;
			else if (PCRNum == 19) return PCR19;
			else return "";
		}
		int getLocality() { return locality; }
		std::string getAuthValue() { return authValue; }
	};

	// abstractPolicy: Provides a protected Secret reference for actual policies 
	class abstractPolicy : public Secret {
	protected:
		Secret * secretObject;
	};

	// policyPCR: Checking the digest of a specified PCR value
	class policyPCR : public abstractPolicy {
	public:
		policyPCR(Secret *secretObject, const std::string& toSet, int PCRNum) : PCRNum(PCRNum) {
			digest = toSet;
			this->secretObject = secretObject;
		}
		std::string getSecret() {
			if (verify() == false) {
				std::cout << "Policy authentiation error on " << PCRNum << std::endl;
				// throw("Policy authentiation error on " + PCRNum);
				return "";
			}
			return secretObject->getSecret();
		}
		std::string getPCR(int num) { return secretObject->getPCR(num); }
		int getLocality() { return secretObject->getLocality(); }
		std::string getAuthValue() { return secretObject->getAuthValue(); }
	private:
		bool verify() {
			if(PCRNum == 17) return (this->secretObject->getPCR(17) == digest);
			else if (PCRNum == 19) return (this->secretObject->getPCR(19) == digest);
			return true;
		}
	private:
		int PCRNum;
		std::string digest;
	};

	// policyLocality: Checking the current locality
	class policyLocality : public abstractPolicy {
	public:
		policyLocality(Secret *secretObject, int toSet) {
			locality = toSet;
			this->secretObject = secretObject;
		}
		std::string getPCR(int num) { return secretObject->getPCR(num); }
		int getLocality() { return secretObject->getLocality(); }
		std::string getAuthValue() { return secretObject->getAuthValue(); }
		std::string getSecret() {
			if (verify() == false) {
				std::cout << "Policy authentiation error on locality" << std::endl;
				// throw("Policy authentiation error on locality");
				return "";
			}
			return secretObject->getSecret();
		}
	private:
		bool verify() {
			return (this->secretObject->getLocality() == locality);
		}
	private:
		int locality;
	};

	// policyAuthValue: Checking the value of authValue
	class policyAuthValue : public abstractPolicy {
	public:
		policyAuthValue(Secret *secretObject, const std::string& toSet) {
			authValue = toSet;
			this->secretObject = secretObject;
		}
		std::string getPCR(int num) { return secretObject->getPCR(num); }
		int getLocality() { return secretObject->getLocality(); }
		std::string getAuthValue() { return secretObject->getAuthValue(); }
		std::string getSecret() {
			if (verify() == false) {
				std::cout << "Policy authentiation error on auth value" << std::endl;
				// throw("Policy authentiation error on auth value");
				return "";
			}
			return secretObject->getSecret();
		}
	private:
		bool verify() {
			return (this->secretObject->getAuthValue() == authValue);
		}
	private:
		std::string authValue;
	};
}
